package net.zenrindatacom.geofenceapp_sample.geofence;

import android.util.Log;

import net.zenrindatacom.geofenceapp_sample.application.SampleApplication;
import net.zenrindatacom.geofenceapp_sample.http.AreaServerException;
import net.zenrindatacom.geofenceapp_sample.http.ServerAccess;
import net.zenrindatacom.geofenceapp_sample.http.TokenServerException;
import net.zenrindatacom.geofenceapp_sample.http.dto.SrvSearchAreaReqDto;
import net.zenrindatacom.geofenceapp_sample.http.dto.SrvSearchAreaResDto;
import net.zenrindatacom.geofenceapp_sample.http.dto.SrvSearchAreaUpdateTimeResDto;
import net.zenrindatacom.geofenceapp_sample.http.dto.SrvTokenReqDto;
import net.zenrindatacom.geofenceapp_sample.http.dto.SrvTokenResDto;
import net.zenrindatacom.geofenceapp_sample.preference.AppPreference;
import net.zenrindatacom.geofencelib.GeoFenceManager;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Date;

/**
 * ジオフェンス管理
 *
 * @author N.Miyahara
 */
public class GeoFenceWrapperManager {

    /** ログ用タグ */
    private static final String TAG = "GeoFenceWrapperManager";

    /** 自インスタンス */
    private static final GeoFenceWrapperManager self = new GeoFenceWrapperManager();

    /** ジオフェンスライブラリのインスタンス（本アプリでの唯一のインスタンス生成箇所） */
    private final GeoFenceManager geoFenceManager = new GeoFenceManager(SampleApplication.getAppContext());

    /** トークン取得APIのレスポンス */
    private SrvTokenResDto srvTokenResDto;

    /** トークン有効期限 */
    private Date tokenLimit = null;

    /** エリア検索APIで1回の通信でサーバから取得する件数の上限（デフォルト値を明示的に定義） */
    private static final int SEARCH_AREA_GET_COUNT_LIMIT = 100;

    /**
     * エリア情報
     */
    public static class AreaInfo {
        /** エリアグループID */
        private String areaGroupId;

        /** エリアID */
        private String areaId;

        /** エリア形状 (Json文字列) */
        private String areaShape;

        public String getAreaGroupId() {
            return areaGroupId;
        }
        public void setAreaGroupId(String areaGroupId) {
            this.areaGroupId = areaGroupId;
        }

        public String getAreaId() {
            return areaId;
        }
        public void setAreaId(String areaId) {
            this.areaId = areaId;
        }

        public String getAreaShape() {
            return areaShape;
        }
        public void setAreaShape(String areaShape) {
            this.areaShape = areaShape;
        }
    }

    /** コンストラクタ（外部からのnew禁止） */
    private GeoFenceWrapperManager() {}

    /** 本クラスのインスタンスを取得 */
    public static GeoFenceWrapperManager getInstance() {
        return self;
    }

    /** ジオフェンスライブラリのインスタンスを取得 */
    public static GeoFenceManager getGeoFenceManager() {
        return self.geoFenceManager;
    }

    /**
     * サーバーから「エリア更新日時」を取得する
     *
     * @return エリア更新日時
     * @throws GeoFenceWrapperException 本クラスの集約例外
     */
    public String getAreaUpdateTimeFromServer() throws GeoFenceWrapperException {
        try {
            //トークン発行サーバからトークンを取得する
            getToken();

            //エリア情報サーバからエリア更新日時を取得する
            return searchAreaUpdateTime();

        } catch(GeoFenceWrapperException ex) {
            Log.d(TAG, "getAreaUpdateTimeFromServer() error.");
            throw ex;
        }
    }

    /**
     * サーバから「エリア」を取得する
     *
     * @return エリア情報リスト
     * @throws GeoFenceWrapperException 本クラスの集約例外
     */
    public ArrayList<AreaInfo> getAreaFromServer() throws GeoFenceWrapperException {

        //サーバから取得するエリアの開始位置（初回=1）
        int offset = 1;
        //サーバに格納されているエリア総数
        int serverAreaCount;
        //取得済のエリア総数
        int areaCount = 0;

        //返却するエリア情報
        ArrayList<AreaInfo> areaInfoArray = new ArrayList<>();

        //1回の通信で取得できる件数に上限があるので、複数回に分けてサーバから取得する
        do {
            SrvSearchAreaResDto.Response.Result resultOne;
            try {
                //トークン発行サーバからトークンを取得する
                getToken();
                //エリア情報サーバからエリア（１回の通信分）を取得する
                resultOne = searchArea(SEARCH_AREA_GET_COUNT_LIMIT, offset);
            } catch (GeoFenceWrapperException ex) {
                Log.d(TAG, "getAreaFromServer() error.");
                throw ex;
            }

            //「今回サーバから取得したエリア情報」を「上位に返すエリア情報」に追加（必要なもののみ返却）
            resultOne.getAreas().forEach( serverArea -> {
                AreaInfo areaInfo = new AreaInfo();
                areaInfo.setAreaGroupId(serverArea.getAreaGroupId());
                areaInfo.setAreaId(serverArea.getAreaId());
                areaInfo.setAreaShape(serverArea.getAreaShape());
                areaInfoArray.add(areaInfo);
            });

            //サーバに格納されているエリア個数
            serverAreaCount = resultOne.getLimitBeforeCount();

            //今回取得したエリア個数
            int thisTimeAreaCount = resultOne.getAreas().size();

            //これまでに取得したエリア総数
            areaCount += thisTimeAreaCount;

            //次リクエストのオフセット
            offset += SEARCH_AREA_GET_COUNT_LIMIT;

        } while(serverAreaCount > areaCount);

        return areaInfoArray;
    }

    /**
     * （トークン発行サーバ）トークン取得
     *
     * @throws GeoFenceWrapperException 本クラスの集約例外
     */
    private void getToken() throws GeoFenceWrapperException {

        //トークンを取得する必要があるか？
        boolean isNeedGetToken;
        if(tokenLimit == null) {
            isNeedGetToken = true;
        } else {
            //トークン有効期限を過ぎているか？
            isNeedGetToken = (new Date()).after(tokenLimit);
        }

        if(!isNeedGetToken) {
            //トークンを取得する必要はない（取得済のトークンを使用する）
            return;
        }

        AppPreference appPreference = AppPreference.getInstance();

        //クライアントID
        final String clientId = appPreference.getClientId();
        //秘密鍵
        final String secretKey = appPreference.getSecret();
        //トークン発行サーバURL
        final String tokenServerUrl = appPreference.getTokenServerUrl();

        SrvTokenReqDto srvTokenReqDto = new SrvTokenReqDto();
        srvTokenReqDto.setGrantType("client_credentials");

        try {
            //トークン取得API実行
            srvTokenResDto = ServerAccess.getInstance().getToken(tokenServerUrl, clientId, secretKey, srvTokenReqDto);
        } catch (TokenServerException ex) {

            GeoFenceWrapperException.ErrorType geoErrorType;

            switch(ex.getErrorType()){
                case TIMEOUT:
                    //タイムアウト
                    geoErrorType = GeoFenceWrapperException.ErrorType.TOKEN_SERVER_TIMEOUT;
                    break;
                case SERVER_AUTH_FAILED:
                    //サーバーエラー（認証失敗）
                    geoErrorType = GeoFenceWrapperException.ErrorType.TOKEN_SERVER_AUTH_FAILED;
                    break;
                case SERVER_ERROR:
                    //サーバエラー(上記以外)
                    geoErrorType = GeoFenceWrapperException.ErrorType.TOKEN_SERVER_ERROR;
                    break;
                case ETC_ERROR:
                default:
                    //ローカル上のエラー
                    geoErrorType = GeoFenceWrapperException.ErrorType.TOKEN_SERVER_LOCAL_ERROR;
                    break;
            }

            throw new GeoFenceWrapperException(geoErrorType, ex);
        }

        //トークン有効期限を記憶する
        Calendar calendar = Calendar.getInstance();
        calendar.setTime(new Date());
        calendar.add(Calendar.SECOND, srvTokenResDto.getExpiresIn());
        tokenLimit = calendar.getTime();
    }

    /**
     * （エリア情報サーバ）エリア更新日時検索
     *
     * @return エリア更新日時
     * @throws GeoFenceWrapperException 本クラスの集約例外
     */
    private String searchAreaUpdateTime() throws GeoFenceWrapperException {

        final String methodName = "searchAreaUpdateTime";

        AppPreference appPreference = AppPreference.getInstance();

        //APIキー
        final String apiKey = appPreference.getApiKey();
        //エリア更新日時検索URL
        final String url = appPreference.getSearchAreaUpdateTimeUrl();
        //トークン
        final String token = srvTokenResDto.getAccessToken();
        //トークンタイプ
        final String tokenType = srvTokenResDto.getTokenType();
        //レスポンス
        SrvSearchAreaUpdateTimeResDto resDto;

        try {
            //エリア更新日時検索API実行
            resDto = ServerAccess.getInstance().searchAreaUpdateTime(url, apiKey, tokenType, token);
        } catch (AreaServerException ex) {

            GeoFenceWrapperException.ErrorType geoErrorType;

            switch(ex.getErrorType()) {
                case TIMEOUT:
                    //タイムアウト
                    geoErrorType = GeoFenceWrapperException.ErrorType.AREA_SERVER_TIMEOUT;
                    break;
                case SERVER_PARAM_ERROR:
                    //サーバーエラー（パラメータエラー）
                case SERVER_ERROR:
                    //サーバエラー(上記以外)
                    geoErrorType = GeoFenceWrapperException.ErrorType.AREA_SERVER_ERROR;
                    break;
                case ETC_ERROR:
                    //ローカル上のエラー
                default:
                    geoErrorType = GeoFenceWrapperException.ErrorType.AREA_SERVER_LOCAL_ERROR;
                    break;
            }

            throw new GeoFenceWrapperException(geoErrorType, ex);
        }

        int returnCode = resDto.getResponse().getStatus().getReturnCode();

        if(returnCode == 0){
            //処理結果コードが正常
            Log.i(TAG, String.format("%s:search-area-update-time API Success. returnCode=%d", methodName, returnCode));
        } else {
            //処理結果コードが異常
            String errorMessage = resDto.getResponse().getStatus().getErrorMessage();
            Log.e(TAG, String.format("%s:search-area-update-time API Fail. returnCode=%d errorMessage=%s", methodName, returnCode, errorMessage));
            throw new GeoFenceWrapperException(GeoFenceWrapperException.ErrorType.AREA_SERVER_ERROR);
        }

        return resDto.getResponse().getResult().getUpdateTime();
    }

    /**
     * （エリア情報サーバ）エリア検索
     *
     * @param limit 取得件数
     * @param offset 取得開始位置
     * @return エリア情報
     * @throws GeoFenceWrapperException 本クラスの集約例外
     */
    private SrvSearchAreaResDto.Response.Result searchArea(@SuppressWarnings("SameParameterValue") final int limit, final int offset) throws GeoFenceWrapperException {

        final String methodName = "searchArea";

        AppPreference appPreference = AppPreference.getInstance();

        //APIキー
        final String apiKey = appPreference.getApiKey();
        //エリア検索URL
        final String url = appPreference.getSearchAreaUrl();

        //トークン
        String token = srvTokenResDto.getAccessToken();
        //トークンタイプ
        String tokenType = srvTokenResDto.getTokenType();

        SrvSearchAreaReqDto reqDto = new SrvSearchAreaReqDto();
        //取得件数
        reqDto.setLimit(limit);
        //取得開始位置
        reqDto.setOffset(offset);

        //レスポンス
        SrvSearchAreaResDto resDto;

        try {
            //エリア検索API実行
            resDto = ServerAccess.getInstance().searchArea(url, apiKey, tokenType, token, reqDto);
        } catch (AreaServerException ex) {

            GeoFenceWrapperException.ErrorType geoErrorType;

            switch(ex.getErrorType()) {
                case TIMEOUT:
                    //タイムアウト
                    geoErrorType = GeoFenceWrapperException.ErrorType.AREA_SERVER_TIMEOUT;
                    break;
                case SERVER_PARAM_ERROR:
                    //サーバーエラー（パラメータエラー）
                case SERVER_ERROR:
                    //サーバエラー(上記以外)
                    geoErrorType = GeoFenceWrapperException.ErrorType.AREA_SERVER_ERROR;
                    break;
                case ETC_ERROR:
                    //ローカル上のエラー
                default:
                    geoErrorType = GeoFenceWrapperException.ErrorType.AREA_SERVER_LOCAL_ERROR;
                    break;
            }

            throw new GeoFenceWrapperException(geoErrorType, ex);
        }

        int returnCode = resDto.getResponse().getStatus().getReturnCode();

        if(returnCode == 0){
            //処理結果コードが正常
            Log.i(TAG, String.format("%s:search-area API Success. returnCode=%d", methodName, returnCode));
        } else {
            //処理結果コードが異常
            String errorMessage = resDto.getResponse().getStatus().getErrorMessage();
            Log.i(TAG, String.format("%s:search-area API Fail. returnCode=%d errorMessage=%s", methodName, returnCode, errorMessage));
            throw new GeoFenceWrapperException(GeoFenceWrapperException.ErrorType.AREA_SERVER_ERROR);
        }

        return resDto.getResponse().getResult();
    }
}
