package net.zenrindatacom.geofenceapp_sample.preference;

import android.content.Context;
import android.content.SharedPreferences;

import net.zenrindatacom.geofenceapp_sample.R;
import net.zenrindatacom.geofenceapp_sample.application.SampleApplication;

/**
 * 「アプリ設定」プリファレンス
 *
 * @author N.Miyahara
 */
public class AppPreference {

    /** プリファレンスが初期化済かを表すキー */
    private static final String IS_PREFERENCE_INITIALIZED = "isInitialized";

    /** プリファレンス「エリア更新日時」キー（アプリ設定画面の項目とは別に独自に保持する） */
    private static final String KEY_AREA_UPDATE_TIME = "AreaUpdateTime";

    /** 本クラスのインスタンス */
    private static final AppPreference self = new AppPreference();

    /** 「アプリ設定」プリファレンス名（ファイル名） */
    private final String preferenceName;

    /** 「アプリ設定」プリファレンスのSharedPreferencesインスタンス */
    private final SharedPreferences sharedPreferences;

    /**
     * コンストラクタ
     */
    private AppPreference() {
        Context ctx = SampleApplication.getAppContext();
        preferenceName = ctx.getPackageName() + "_app_setting_preferences";
        sharedPreferences = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
    }

    /**
     * 本クラスのインスタンスを取得する
     * @return 本クラスのインスタンス
     */
    public static AppPreference getInstance() {
        return self;
    }

    /**
     * 「アプリ設定」プリファレンス名（ファイル名）を取得する
     * @return プリファレンス名
     */
    public String getPreferenceName() {
        return preferenceName;
    }

    /**
     * 「アプリ設定」プリファレンス初期化
     * （プリファレンスが作成されてなければ初期データを設定する）
     */
    public void init() {

        if (sharedPreferences.getBoolean(IS_PREFERENCE_INITIALIZED, false)) {
            //初期化済なら何もしない
            return;
        }

        Context ctx = SampleApplication.getAppContext();

        SharedPreferences.Editor editor = sharedPreferences.edit();

        //クライアントID
        editor.putString(ctx.getString(R.string.prf_app_key_client_id), "");
        //秘密鍵
        editor.putString(ctx.getString(R.string.prf_app_key_secret), "");
        //トークン発行サーバURL
        editor.putString(ctx.getString(R.string.prf_app_key_token_server_url), "");
        //APIキー
        editor.putString(ctx.getString(R.string.prf_app_key_api_key), "");
        //（エリア情報サーバー）エリア更新日時検索APIのURL
        editor.putString(ctx.getString(R.string.prf_app_key_search_area_update_time_url), "");
        //（エリア情報サーバー）エリア検索APIのURL
        editor.putString(ctx.getString(R.string.prf_app_key_search_area_url), "");

        //プリファレンスに初期データを設定済
        editor.putBoolean(IS_PREFERENCE_INITIALIZED, true);

        editor.apply();
    }

    /**
     * 「アプリ設定」プリファレンスを初期データに戻す
     */
    public void returnToDefault() {
        sharedPreferences.edit().putBoolean(IS_PREFERENCE_INITIALIZED, false).apply();
        init();
    }

    /**
     * プリファレンス値「クライアントID」の取得
     * @return プリファレンス値「クライアントID」
     */
    public String getClientId() {
        Context ctx = SampleApplication.getAppContext();
        return sharedPreferences.getString(ctx.getString(R.string.prf_app_key_client_id), "");
    }

    /**
     * プリファレンス値「秘密鍵」の取得
     * @return プリファレンス値「秘密鍵」
     */
    public String getSecret() {
        Context ctx = SampleApplication.getAppContext();
        return sharedPreferences.getString(ctx.getString(R.string.prf_app_key_secret), "");
    }

    /**
     * プリファレンス値「トークン発行サーバURL」の取得
     * @return プリファレンス値「トークン発行サーバURL」
     */
    public String getTokenServerUrl() {
        Context ctx = SampleApplication.getAppContext();
        return sharedPreferences.getString(ctx.getString(R.string.prf_app_key_token_server_url), "");
    }

    /**
     * プリファレンス値「APIキー」の取得
     * @return プリファレンス値「APIキー」
     */
    public String getApiKey() {
        Context ctx = SampleApplication.getAppContext();
        return sharedPreferences.getString(ctx.getString(R.string.prf_app_key_api_key), "");
    }

    /**
     * プリファレンス値「（エリア情報サーバー）エリア更新日時検索APIのURL」の取得
     * @return プリファレンス値「（エリア情報サーバー）エリア更新日時検索APIのURL」
     */
    public String getSearchAreaUpdateTimeUrl() {
        Context ctx = SampleApplication.getAppContext();
        return sharedPreferences.getString(ctx.getString(R.string.prf_app_key_search_area_update_time_url), "");
    }

    /**
     * プリファレンス値「（エリア情報サーバー）エリア検索APIのURL」の取得
     * @return プリファレンス値「（エリア情報サーバー）エリア検索APIのURL」
     */
    public String getSearchAreaUrl() {
        Context ctx = SampleApplication.getAppContext();
        return sharedPreferences.getString(ctx.getString(R.string.prf_app_key_search_area_url), "");
    }

    /**
     * プリファレンス値「エリア更新日時」の取得
     * @return プリファレンス値「エリア更新日時」
     */
    public String getAreaUpdateTime() {
        return sharedPreferences.getString(KEY_AREA_UPDATE_TIME, "");
    }

    /**
     * プリファレンス値「エリア更新日時」の設定
     * @param value エリア更新日時
     */
    public void setAreaUpdateTime(String value) {
        sharedPreferences.edit().putString(KEY_AREA_UPDATE_TIME, value).apply();
    }
}
