package net.zenrindatacom.geofenceapp_sample.preference;

import android.content.Context;
import android.content.SharedPreferences;

import net.zenrindatacom.geofenceapp_sample.R;
import net.zenrindatacom.geofenceapp_sample.application.SampleApplication;
import net.zenrindatacom.geofencelib.GeoFenceManager;

/**
 * 「ジオフェンスライブラリ設定」プリファレンス
 *
 * @author N.Miyahara
 */
public class GeoFenceLibPreference {

    /** プリファレンスが初期化済かを表すキー */
    private static final String IS_PREFERENCE_INITIALIZED = "isInitialized";

    /** 本クラスのインスタンス */
    private static final GeoFenceLibPreference self = new GeoFenceLibPreference();

    /** 「ジオフェンスライブラリ設定」プリファレンス名（ファイル名） */
    private final String preferenceName;

    /** 「ジオフェンスライブラリ設定」プリファレンスのSharedPreferencesインスタンス */
    private final SharedPreferences sharedPreferences;

    /**
     * コンストラクタ
     */
    private GeoFenceLibPreference() {
        Context ctx = SampleApplication.getAppContext();
        preferenceName = ctx.getPackageName() + "_geofence_lib_setting_preferences";
        sharedPreferences = ctx.getSharedPreferences(preferenceName, Context.MODE_PRIVATE);
    }

    /**
     * 本クラスのインスタンスを取得する
     * @return 本クラスのインスタンス
     */
    public static GeoFenceLibPreference getInstance() {
        return self;
    }

    /**
     * 「ジオフェンスライブラリ設定」プリファレンス名（ファイル名）を取得する
     * @return プリファレンス名
     */
    public String getPreferenceName() {
        return preferenceName;
    }

    /**
     * 「ジオフェンスライブラリ設定」プリファレンス初期化
     * （プリファレンスが作成されてなければ初期データを設定する）
     */
    public void init() {

        if (sharedPreferences.getBoolean(IS_PREFERENCE_INITIALIZED, false)) {
            //初期化済なら何もしない
            return;
        }

        Context ctx = SampleApplication.getAppContext();

        SharedPreferences.Editor editor = sharedPreferences.edit();

        //APIキー
        editor.putString(ctx.getString(R.string.prf_geofence_lib_key_api_key), "");
        //クライアントID
        editor.putString(ctx.getString(R.string.prf_geofence_lib_key_client_id), "");
        //秘密鍵
        editor.putString(ctx.getString(R.string.prf_geofence_lib_key_secret), "");
        //認証サーバ環境
        editor.putString(ctx.getString(R.string.prf_geofence_lib_key_auth_env), "0");

        //プリファレンスに初期データを設定済
        editor.putBoolean(IS_PREFERENCE_INITIALIZED, true);

        editor.apply();
    }

    /**
     * 「ジオフェンスライブラリ設定」プリファレンスを初期データに戻す
     */
    public void returnToDefault() {
        sharedPreferences.edit().putBoolean(IS_PREFERENCE_INITIALIZED, false).apply();
        init();
    }

    /**
     * プリファレンス値「APIキー」の取得
     * @return プリファレンス値「APIキー」
     */
    public String getApiKey() {
        Context ctx = SampleApplication.getAppContext();
        return sharedPreferences.getString(ctx.getString(R.string.prf_geofence_lib_key_api_key), "");
    }

    /**
     * プリファレンス値「クライアントID」の取得
     * @return プリファレンス値「クライアントID」
     */
    public String getClientId() {
        Context ctx = SampleApplication.getAppContext();
        return sharedPreferences.getString(ctx.getString(R.string.prf_geofence_lib_key_client_id), "");
    }

    /**
     * プリファレンス値「秘密鍵」の取得
     * @return プリファレンス値「秘密鍵」
     */
    public String getSecret() {
        Context ctx = SampleApplication.getAppContext();
        return sharedPreferences.getString(ctx.getString(R.string.prf_geofence_lib_key_secret), "");
    }

    /**
     * プリファレンス値「認証サーバ環境」の取得
     * @return プリファレンス値「認証サーバ環境」
     */
    public GeoFenceManager.AuthEnv getAuthEnv() {
        Context ctx = SampleApplication.getAppContext();
        String value = sharedPreferences.getString(ctx.getString(R.string.prf_geofence_lib_key_auth_env), "0");
        switch (Integer.parseInt(value)) {
            case 0:
                return GeoFenceManager.AuthEnv.AUTH_ENV_PRO;
            case 1:
                return GeoFenceManager.AuthEnv.AUTH_ENV_STG;
            case 2:
                return GeoFenceManager.AuthEnv.AUTH_ENV_TEST;
            default:
                return GeoFenceManager.AuthEnv.AUTH_ENV_PRO;
        }
    }
}
